﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;

using Microsoft.SiteAnalyzer.Core;

namespace Microsoft.SiteAnalyzer.SiteMapBuild
{
    class Program
    {
        static List<PageInfo> _pageInfoList = new List<PageInfo>();
        static PageGlobalFeature _pageGlobalFeature = new PageGlobalFeature();
        
        [STAThread]
        static void Main(string[] args)
        {
            if (args.Length != 1)
            {
                PrintUsage();
                return;
            }

            string dataDir = args[0];
            if (!Directory.Exists(dataDir))
            {
                Console.WriteLine("Cannot find the directory {0}", dataDir);
                PrintUsage();
                return;
            }

            PageInfoExtraction(dataDir);

            CalculateGlobalFeature(dataDir);

            BuildSiteMap(dataDir);
        }

        static void PageInfoExtraction(string dataDir)
        {
            PageInfoParser pageInfoParser = new PageInfoParser();
            _pageInfoList.Clear();

            int processed = 0;
            using (StreamReader sr = new StreamReader(dataDir + @"\meta.txt"))
            { 
                while (!sr.EndOfStream)
                {
                    string[] paras = sr.ReadLine().Split("\t".ToCharArray());
                    string filePath = dataDir + @"\" + paras[0] + @".htm";

                    PageInfo page = new PageInfo();
                    page.Url = paras[1];
                    using (StreamReader contentReader = new StreamReader(filePath))
                    {
                        string content = contentReader.ReadToEnd();
                        content = content.Replace("\r", string.Empty);
                        content = content.Replace("\n", string.Empty);
                        if (!String.IsNullOrEmpty(content))
                        {
                            try
                            {
                                if (pageInfoParser.ParsePageInfo(page, content) == true)
                                {
                                    _pageInfoList.Add(page);
                                }
                            }
                            catch (Exception ex)
                            {
                                Console.WriteLine(ex.Message);
                                Console.WriteLine(ex.StackTrace);
                            }
                        }
                    }

                    if (++processed % 100 == 0)
                        Console.WriteLine("{0} pages processed", processed);
                }
            }
            PageInfoIOHelper.SavePageInfoList(_pageInfoList, dataDir + @"\PageInfo.xml");
        }

        static void CalculateGlobalFeature(string dataDir)
        {
            _pageGlobalFeature.CalculateGlobalFeature(_pageInfoList);
            PageGlobalFeatureIOHelper.SaveGlobalFeatures(_pageGlobalFeature, dataDir + @"\PageGlobalFeature.xml");
        }

        static void BuildSiteMap(string dataDir)
        {
            SiteMap siteMap = SiteMapBuilder.BuildSiteMap(_pageGlobalFeature, _pageInfoList);
            SiteMapIOHelper.SaveSiteMap(_pageGlobalFeature, siteMap, dataDir + @"\SiteMap.xml");
            SiteMapIOHelper.SaveSiteMapClusters(siteMap, dataDir + @"\SiteMapCluster.xml", _pageInfoList);
        }

        static void PrintUsage()
        {
            Console.WriteLine("SiteAnalyzer Utility - Sitemap Builder");
            Console.WriteLine("Description: to build a sitemap of a given site.");
            Console.WriteLine("====================================================");
            Console.WriteLine("Command line arguments : >Microsoft.SiteAnalyzer.SiteMapBuilder [0]");
            Console.WriteLine("[0] : The directory path of the page collection.");
            Console.WriteLine("====================================================");
        }

    }
}
